# Generated by Django 2.1.3 on 2018-11-14 15:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geonames_place', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='place',
            name='geonames_id',
            field=models.PositiveIntegerField(unique=True),
        ),
    ]



from django.contrib import admin
from geonames_place.models import (ClassDescription, Country, FeatureClass,
                                   Place)


@admin.register(ClassDescription)
class ClassDescriptionAdmin(admin.ModelAdmin):
    search_fields = ['title']


@admin.register(Country)
class CountryAdmin(admin.ModelAdmin):
    search_fields = ['name', 'code']


@admin.register(FeatureClass)
class FeatureClassAdmin(admin.ModelAdmin):
    search_fields = ['title']


@admin.register(Place)
class PlaceAdmin(admin.ModelAdmin):
    autocomplete_fields = ['class_description', 'country', 'feature_class']
    search_fields = ['address', 'country__name', 'country__code']



# Generated by Django 2.1.3 on 2018-11-15 11:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geonames_place', '0002_alter_field_geonames_id_on_place'),
    ]

    operations = [
        migrations.AddField(
            model_name='place',
            name='update_from_geonames',
            field=models.BooleanField(default=True),
        ),
    ]



# -*- coding: utf-8 -*-
import geocoder
from django.conf import settings
from django.db import models
from model_utils.models import TimeStampedModel


class ClassDescription(TimeStampedModel):
    title = models.CharField(max_length=128, unique=True)

    class Meta:
        ordering = ['title']

    def __str__(self):
        return self.title


class Country(TimeStampedModel):
    name = models.CharField(max_length=64, unique=True)
    code = models.CharField(max_length=16, unique=True)

    class Meta:
        ordering = ['name', 'code']
        verbose_name_plural = 'Countries'

    def __str__(self):
        return '{} ({})'.format(self.name, self.code)


class FeatureClass(TimeStampedModel):
    title = models.CharField(max_length=16, unique=True)

    class Meta:
        ordering = ['title']

    def __str__(self):
        return self.title


class Place(TimeStampedModel):
    geonames_id = models.PositiveIntegerField(unique=True)
    update_from_geonames = models.BooleanField(default=True)
    address = models.CharField(max_length=512, blank=True, null=True)
    class_description = models.ForeignKey(
        ClassDescription, blank=True, null=True, on_delete=models.CASCADE)
    country = models.ForeignKey(
        Country, blank=True, null=True, on_delete=models.CASCADE)
    feature_class = models.ForeignKey(
        FeatureClass, blank=True, null=True, on_delete=models.CASCADE)
    lat = models.DecimalField(
        max_digits=9, decimal_places=6, blank=True, null=True)
    lon = models.DecimalField(
        max_digits=9, decimal_places=6, blank=True, null=True)

    class Meta:
        ordering = ['address', 'country']

    def __str__(self):
        return '{}, {}'.format(self.address, self.country)

    def save(self, *args, **kwargs):
        if self.update_from_geonames:
            self.hydrate_from_geonames()
            self.update_from_geonames = False

        super().save(*args, **kwargs)

    def hydrate_from_geonames(self):
        if not self.geonames_id:
            return

        g = geocoder.geonames(
            self.geonames_id, key=settings.GEONAMES_KEY, method='details')

        self._hydrate(g)

    def _hydrate(self, geoname):
        if not geoname:
            return

        self.address = geoname.address

        if geoname.class_description:
            cd, _ = ClassDescription.objects.get_or_create(
                title=geoname.class_description)
            self.class_description = cd

        if geoname.country and geoname.country_code:
            c, _ = Country.objects.get_or_create(
                name=geoname.country, code=geoname.country_code)
            self.country = c

        if geoname.feature_class:
            fc, _ = FeatureClass.objects.get_or_create(
                title=geoname.feature_class)
            self.feature_class = fc

        self.lat = geoname.lat

        if 'lon' in geoname.fieldnames:
            self.lon = geoname.lon
        else:
            self.lon = geoname.lng

    @staticmethod
    def create_or_update_from_geonames(address):
        count = 0

        if not address:
            return count

        if len(address) < 3:
            return count

        geonames = geocoder.geonames(
            address, key=settings.GEONAMES_KEY,
            maxRows=settings.GEONAMES_MAX_RESULTS)

        for g in geonames:
            p, _ = Place.objects.get_or_create(geonames_id=g.geonames_id)
            p._hydrate(g)
            p.save()

            count += 1

        return count



# Generated by Django 2.1.3 on 2018-11-14 10:43

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ClassDescription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('title', models.CharField(max_length=128, unique=True)),
            ],
            options={
                'ordering': ['title'],
            },
        ),
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=64, unique=True)),
                ('code', models.CharField(max_length=16, unique=True)),
            ],
            options={
                'ordering': ['name', 'code'],
            },
        ),
        migrations.CreateModel(
            name='FeatureClass',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('title', models.CharField(max_length=16, unique=True)),
            ],
            options={
                'ordering': ['title'],
            },
        ),
        migrations.CreateModel(
            name='Place',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('geonames_id', models.PositiveIntegerField()),
                ('address', models.CharField(blank=True, max_length=512, null=True)),
                ('lat', models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True)),
                ('lon', models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True)),
                ('class_description', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames_place.ClassDescription')),
                ('country', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames_place.Country')),
                ('feature_class', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames_place.FeatureClass')),
            ],
            options={
                'ordering': ['address', 'country'],
            },
        ),
    ]
